"""Tests for replace_render_to_response script"""
import subprocess

import os
import shutil


def setup_local_copy(tmpdir, path):
    sample_file = os.path.join(os.path.dirname(__file__), "sample_files", path)
    temp_file_path = str(tmpdir.join('sample_render_to_response_tmp.py'))
    shutil.copy2(sample_file, temp_file_path)
    return temp_file_path


def test_replace_script(tmpdir):
    """
    Test replace script on a file to make sure it is renaming it properly and
    also adding request parameter to updated function call
    """
    sample_file_path = setup_local_copy(tmpdir, "sample_render_to_response.py")
    with open(sample_file_path) as sample_file:
        sample_code = sample_file.read()
        assert "render_to_response" in sample_code
        assert "render_to_response(request, " not in sample_code
        assert "render(request, " not in sample_code
    subprocess.call(['replace_render_to_response', sample_file_path])
    with open(sample_file_path) as sample_file:
        sample_code = sample_file.read()
        assert "render_to_response" not in sample_code
        assert "render" in sample_code
        # Adding request parameter to function call
        assert "render(request, " in sample_code


def test_replace_script_avoid_non_django_version(tmpdir):
    """
    Test replace script on the file in which function name shouldn't be changed
    as it is being imported from some local directory instead of django.shortcuts
    """
    sample_file_path = setup_local_copy(tmpdir, "sample_render_to_response_2.py")
    with open(sample_file_path) as sample_file:
        sample_code = sample_file.read()
        assert "render_to_response" in sample_code
        assert "render(request, " not in sample_code
    subprocess.call(['replace_render_to_response', sample_file_path])
    with open(sample_file_path) as sample_file:
        sample_code = sample_file.read()
        # Shouldn't replace render_to_response in this file
        assert "render_to_response" in sample_code
        assert "render(request, " not in sample_code



"""
Tests for verifying the output of the script edx_repo_tools/codemods/django3/remove_python2_unicode_compatible.py
"""
import os
from unittest import TestCase
import shutil
import uuid
import subprocess
from edx_repo_tools.codemods.django3 import remove_python2_unicode_compatible


def setup_local_copy(tmpdir):
    """
    Setup local copy of the sample file for tests
    """
    sample_test_file = os.path.join(os.path.dirname(__file__), "sample_files/sample_python2_unicode_removal.py")
    temp_file_path = str(tmpdir.join('test-python2-unicode.py'))
    shutil.copy2(sample_test_file, temp_file_path)
    return temp_file_path


def test_python2_unicode_compatible_removed(tmpdir):
    """
    Test the script on a sample file to make sure it is removing python_2_unicode_compatible imports and decorators.
    """
    sample_file_path = setup_local_copy(tmpdir)
    with open(sample_file_path) as sample_file:
        sample_code = sample_file.read()
        assert "python_2_unicode_compatible" in sample_code
    remove_python2_unicode_compatible.run_removal_query(sample_file_path)
    with open(sample_file_path) as sample_file:
        sample_code = sample_file.read()
        assert "python_2_unicode_compatible" not in sample_code



"""
Collect and report on conventional commit statistics.
"""

import csv
import datetime
import fnmatch
import os
import os.path
import re
import sqlite3
import sys

import click
try:
    import dataset
    import matplotlib.pyplot as plt
    import matplotlib.dates
    import pandas as pd
except ImportError as err:
    sys.exit(f"Did you install requirements/conventional_commits.txt? {err}")

from edx_repo_tools.utils import change_dir, get_cmd_output


@click.group(help=__doc__)
def main():
    pass

def load_commits(db, repo_name):
    """Load the commits from the current directory repo."""

    SEP = "-=:=-=:=-=:=-=:=-=:=-=:=-=:=-"
    GITLOG = f"git log --no-merges --format='format:date: %aI%nhash: %H%nauth: %aE%nname: %aN%nsubj: %s%n%b%n{SEP}'"
    SHORT_LINES = 5

    # -=:=-=:=-=:=-=:=-=:=-=:=-=:=-
    # date: 2021-07-06T15:51:32-04:00
    # hash: ecd257ae297277ef4e544e44f4e803dfd48f238c
    # auth: JHynes@edx.org
    # name: Justin Hynes
    # subj: feat!: Remove temp certificates mgmt cmd
    # [MICROBA-1311]
    # - Remove temporary management command used to fix records incorrectly created with a default `mode` of "honor".
    #
    # -=:=-=:=-=:=-=:=-=:=-=:=-=:=-
    # date: 2021-07-06T12:48:21-04:00
    # hash: 384bc6b5147423f9c3208ce5c4afa79e5e0cd040
    # auth: 8483753+crice100@users.noreply.github.com
    # name: Christie Rice
    # subj: fix: Fix cert status (#28097)
    # MICROBA-1372
    # -=:=-=:=-=:=-=:=-=:=-=:=-=:=-

    with db:
        commit_table = db["commits"]

        log = get_cmd_output(GITLOG)
        for i, commit in enumerate(log.split(SEP + "\n")):
            if commit:
                lines = commit.split("\n", maxsplit=SHORT_LINES)
                row = {"repo": repo_name}
                for line in lines[:SHORT_LINES]:
                    key, val = line.split(": ", maxsplit=1)
                    row[key] = val
                row["body"] = lines[SHORT_LINES].strip()
                analyze_commit(row)
                commit_table.insert(row)

# Strict conformance to OEP-51.
STRICT = r"""(?x)
    ^
    (?P<label>build|chore|docs|feat|fix|perf|refactor|revert|style|test|temp)
    (?P<breaking>!?):\s
    (?P<subjtext>.+)
    $
    """

# Looser checking of conformance to conventional commits.
LAX = r"""(?xi) # case-insensitive
    ^
    # some labels can be pluralized, since it's hard to remember.
    (?P<label>build|chores?|docs?|feat|fix|perf|refactor|revert|style|tests?|temp)
    # an optional scope is allowed
    (?:\(\w+\))?
    (?P<breaking>!?):\s
    (?P<subjtext>.+)
    $
    |
    # GitHub revert PR commit syntax
    ^Revert\s+"(?P<subjtext2>.+)"(?:\s+\(\#\d+\))?$
    """

def analyze_commit(row):
    row["conventional"] = row["lax"] = False
    m = re.search(STRICT, row["subj"])
    if m:
        row["conventional"] = True
    else:
        m = re.search(LAX, row["subj"])
        if m:
            row["lax"] = True
    if m:
        row["label"] = m["label"]
        row["breaking"] = bool(m["breaking"])
        row["subjtext"] = m["subjtext"] or m["subjtext2"] or ""
    row["bodylines"] = len(row["body"].splitlines())


@main.command(help="Collect stats about commits in local git repos")
@click.option("--db", "dbfile", default="commits.db", help="SQLite database file to write to")
@click.option("--ignore", multiple=True, help="Repos to ignore")
@click.option("--require", help="A file that must exist to process the repo")
@click.argument("repos", nargs=-1)
def collect(dbfile, ignore, require, repos):
    db = dataset.connect("sqlite:///" + dbfile, sqlite_wal_mode=False)
    for repo in repos:
        if any(fnmatch.fnmatch(repo, pat) for pat in ignore):
            print(f"Ignoring {repo}")
            continue
        if require is not None:
            if not os.path.exists(os.path.join(repo, require)):
                print(f"Skipping {repo}")
                continue
        print(repo)
        with change_dir(repo) as repo_dir:
            repo_name = "/".join(repo_dir.split("/")[-2:])
            load_commits(db, repo_name)

    # Write repo->squad mapping to the db.
    with open("edx/repo-health-data/dashboards/dashboard_main.csv") as repos_csv:
        repos_table = db["repos"]
        for row in csv.DictReader(repos_csv):
            repos_table.insert({
                "repo": row["repo_name"],
                "squad": row["ownership.theme"] + "/" + row["ownership.squad"],
            })


QUERY = """\
    select
    weekend, total,
    con, cast((con*100.0)/total as integer) pctcon,
    lax, cast(((con+lax)*100.0)/total as integer) pctlax,
    bod, cast((bod*100.0)/total as integer) pctbod
    from (
        select
        strftime("%Y%m%d", date, "weekday 0") as weekend,
        count(*) total,
        sum(conventional) as con, sum(lax) as lax, sum(bodylines > 0) as bod
        from commits group by weekend
    )
    where weekend > '202009';
    """

@main.command(help="Plot the collected statistics")
def plot():
    # Read sqlite query results into a pandas DataFrame
    with sqlite3.connect("commits.db") as con:
        df = pd.read_sql_query(QUERY, con)

    # Make the date nice
    df["when"] = pd.to_datetime(df["weekend"], format="%Y%m%d")
    # Drop the last row, because it's probably incomplete
    df = df[:-1]

    fig, ax = plt.subplots()
    fig.set_size_inches(12, 8)
    ax.xaxis.set_major_formatter(matplotlib.dates.DateFormatter('%b'))

    lines = []
    lines.append(ax.plot(df.when, df.total, "*-", label="# Commits", color="gray", linewidth=1)[0])

    subplot = ax.twinx()
    subplot.set_ylim(-5, 105)
    lines.append(subplot.plot(df.when, df.pctcon, label="% Strict", color="red", linewidth=4)[0])

    subplot = ax.twinx()
    subplot.set_ylim(-5, 105)
    lines.append(subplot.plot(df.when, df.pctlax, label="% Lax", color="green", linewidth=4)[0])

    subplot = ax.twinx()
    subplot.set_ylim(-5, 105)
    lines.append(subplot.plot(df.when, df.pctbod, label="% with bodies", color="blue", linewidth=2)[0])

    plt.legend(lines, [l.get_label() for l in lines])
    plt.show()

# A by-squad query to try:
#
#   select
#   squad, repos, commits, con, cast((con*100.0)/commits as integer) pctcon
#   from (
#       select
#           r.squad squad,
#           count(distinct r.repo) repos,
#           count(*) commits,
#           sum(conventional) con
#       from commits c, repos r
#       on c.repo = r.repo
#       where date > '2021-06-01'
#       group by 1
#   );

if __name__ == "__main__":
    main()



"""Tests for TOX modernizer """
import os
import re
from configparser import NoSectionError, ConfigParser
from unittest import TestCase
import shutil
import uuid
from edx_repo_tools.codemods.django3 import ConfigReader


class TestToxModernizer(TestCase):
    def setUp(self):
        self.config_file1 = self._setup_local_copy("sample_tox_config.ini")
        self.config_file2 = self._setup_local_copy("sample_tox_config_2.ini")

    @staticmethod
    def _setup_local_copy(file_name):
        current_directory = os.path.dirname(__file__)
        temp_file = os.path.join(current_directory, str(uuid.uuid4()) + ".ini")
        local_file = os.path.join(current_directory, file_name)
        shutil.copy2(local_file, temp_file)
        return temp_file

    def _get_parser(self, file_path):
        modernizer = ConfigReader(file_path=file_path).get_modernizer()
        modernizer.modernize()
        parser = ConfigParser()
        parser.read(file_path)
        self._assert_no_tabs(file_path)
        return parser

    def _assert_django_dependencies_replaced(self, config_file):
        parser = self._get_parser(config_file)
        dependencies = parser['testenv']['deps']

        self.assertNotIn("django111:", dependencies)
        self.assertNotIn("django20:", dependencies)
        self.assertNotIn("django21:", dependencies)
        self.assertIn("django22:", dependencies)
        self.assertIn("django30:", dependencies)
        self.assertIn("django31:", dependencies)
        self.assertIn("django32:", dependencies)

    def _assert_replaces_python_interpreters(self, config_file):
        parser = self._get_parser(config_file)
        env_list = parser['tox']['envlist']

        self.assertNotRegex("py{27}", env_list)
        self.assertNotIn("py{27,35}", env_list)
        self.assertNotIn("py{27,35,36}", env_list)
        self.assertNotIn("py{27,35,36,37}", env_list)
        self.assertIn("py38", env_list)

    def _assert_replaces_django_runners(self, config_file):
        parser = self._get_parser(config_file)
        env_list = parser['tox']['envlist']

        self.assertNotIn("django{111}", env_list)
        self.assertNotIn("django{111,20}", env_list)
        self.assertNotIn("django{111,20,21}", env_list)
        self.assertIn("django{22,30,31,32}", env_list)

    def _assert_replaces_django_dependencies(self, config_file):
        self._assert_django_dependencies_replaced(config_file)

    def _assert_adds_django_dependencies(self, config_file):
        parser = ConfigParser()
        parser.read(config_file)

        dependencies = parser['testenv']['deps']
        dependencies = re.sub("[^\n]*django22.*\n", '', dependencies)
        parser['testenv']['deps'] = dependencies

        with open(config_file, 'w') as configfile:
            parser.write(configfile)

        self._assert_django_dependencies_replaced(config_file)

    def _assert_no_tabs(self, config_file):
        with open(config_file) as configfile:
            assert "\t" not in configfile.read()

    def test_raises_error_no_empty_config(self):
        tox_config = {}
        self.assertRaises(NoSectionError, ConfigReader(config_dict=tox_config).get_modernizer)

    def test_raises_error_bad_config(self):
        tox_config = {'section1': {'key1': 'value1', 'key2': 'value2', 'key3': 'value3'},
                      'section2': {'keyA': 'valueA', 'keyB': 'valueB', 'keyC': 'valueC'},
                      'section3': {'foo': 'x', 'bar': 'y', 'baz': 'z'}}

        self.assertRaises(NoSectionError, ConfigReader(tox_config).get_modernizer)

    def test_replaces_python_interpreters(self):
        self._assert_replaces_python_interpreters(self.config_file1)
        self._assert_replaces_python_interpreters(self.config_file2)

    def test_replaces_django_runners(self):
        self._assert_replaces_django_runners(self.config_file1)
        self._assert_replaces_django_runners(self.config_file2)

    def test_django_dependency_replaced(self):
        self._assert_django_dependencies_replaced(self.config_file1)
        self._assert_django_dependencies_replaced(self.config_file2)

    def test_adds_django_dependency(self):
        self._assert_adds_django_dependencies(self.config_file1)
        self._assert_adds_django_dependencies(self.config_file2)

    def tearDown(self):
        os.remove(self.config_file1)
        os.remove(self.config_file2)



import sys

import click
from bowler import Query


def replace_unicode(path):
    """
    Run the bowler query on the input files for refactoring.
    """
    (
        Query(path)
        .select_function("__unicode__")
        .rename('__str__')
        .idiff()
    ),
    (
        Query(path)
        .select_method("__unicode__")
        .is_call()
        .rename('__str__')
        .idiff()
    )


@click.command()
@click.option('--path', help='use syntax: --path {path_to_input_file/directory}')
def main(path):
    """
    Function to handle input path for refactoring.
    HOW_TO_USE: when running as a repo tool, use following syntax to run the command:
        replace_unicode_with_str --path {path_to_input_file/directory}
    """
    replace_unicode(path)


if __name__ == '__main__':
    main()




import os
import re
import shutil
import uuid
from unittest import TestCase

from edx_repo_tools.codemods.django3 import TravisModernizer, DJANGO_PATTERN
from edx_repo_tools.utils import YamlLoader


class TestTravisModernizer(TestCase):

    def setUp(self):
        self.test_file1 = self._setup_local_copy("test_travis.yml")
        self.test_file2 = self._setup_local_copy("test_travis_2.yml")

    @staticmethod
    def _setup_local_copy(file_name):
        current_directory = os.path.dirname(__file__)
        temp_file = os.path.join(current_directory, str(uuid.uuid4()) + ".yml")
        local_file = os.path.join(current_directory, file_name)
        shutil.copy2(local_file, temp_file)
        return temp_file

    @staticmethod
    def _get_updated_yaml_elements(file_path):
        modernizer = TravisModernizer(file_path)
        modernizer.modernize()
        yaml_loader = YamlLoader(file_path)
        return yaml_loader.elements

    def test_python_env_items(self):
        travis_elements = TestTravisModernizer._get_updated_yaml_elements(self.test_file1)
        python_versions = travis_elements.get("python")

        self.assertIsInstance(python_versions, list)
        self.assertTrue(len(python_versions), 1)
        python_version = python_versions[0]

        self.assertEqual(str(python_version), '3.8')

    def test_python_matrix_items(self):
        travis_elements = TestTravisModernizer._get_updated_yaml_elements(self.test_file2)
        python_versions = [matrix_item for matrix_item in travis_elements.get("matrix").get("include")
                           if 'python' in matrix_item.keys()]

        self.assertIsInstance(python_versions, list)
        self.assertTrue(len(python_versions), 1)
        python_version = python_versions[0].get('python')

        self.assertEqual(python_version, '3.8')

    def test_django_env_items(self):
        travis_elements = TestTravisModernizer._get_updated_yaml_elements(self.test_file1)
        django_envs = [django_env_item for django_env_item in travis_elements.get("env")
                       if re.search(DJANGO_PATTERN, django_env_item)]
        self.assertIsInstance(django_envs, list)
        self.assertTrue(len(django_envs), 3)

        self.assertTrue(any("django22" in django_env for django_env in django_envs))
        self.assertTrue(any("django30" in django_env for django_env in django_envs))
        self.assertTrue(any("django31" in django_env for django_env in django_envs))
        self.assertTrue(any("django32" in django_env for django_env in django_envs))

    def test_django_matrix_items(self):
        travis_elements = TestTravisModernizer._get_updated_yaml_elements(self.test_file2)
        django_matrix_envs = [matrix_item for matrix_item in travis_elements.get("matrix").get("include")
                              if re.search(DJANGO_PATTERN, matrix_item.get("env"))]

        self.assertIsInstance(django_matrix_envs, list)
        self.assertTrue(len(django_matrix_envs), 3)

        django_envs = [matrix_item.get("env") for matrix_item in django_matrix_envs]

        self.assertTrue(any("django22" in django_env for django_env in django_envs))
        self.assertTrue(any("django30" in django_env for django_env in django_envs))
        self.assertTrue(any("django31" in django_env for django_env in django_envs))
        self.assertTrue(any("django32" in django_env for django_env in django_envs))

    def tearDown(self):
        os.remove(self.test_file1)
        os.remove(self.test_file2)



import os
import shutil

from ruamel.yaml import YAML

from edx_repo_tools.modernize_openedx_yaml import YamlModernizer


def setup_local_copy(tmpdir):
    sample_yam_file = os.path.join(os.path.dirname(__file__), "sample_openedx.yaml")
    temp_file_path = str(tmpdir.join('test-openedx.yaml'))
    shutil.copy2(sample_yam_file, temp_file_path)
    return temp_file_path


def load_yaml(sample_yam_file):
    with open(sample_yam_file) as file_stream:
        return YAML().load(file_stream)


def test_travis_modernizer(tmpdir):
    test_yaml_file = setup_local_copy(tmpdir)
    modernizer = YamlModernizer(test_yaml_file)
    modernizer.modernize()
    updated_yaml = load_yaml(test_yaml_file)
    assert 'owner' not in updated_yaml.keys()
    assert 'supporting_teams' not in updated_yaml.keys()
    assert 'track_pulls' not in updated_yaml.keys()
    assert 'track-pulls' not in updated_yaml.keys()



"""
A script to remove the python2_unicode_compatible imports and headers
"""
import sys
import click
from bowler import Query


def remove_node(node, _, __):
    """
    Remove the node containing the expression python_2_unicode_compatible
    """
    node.remove()


def run_removal_query(path):
    """
    Run the bowler query on the input files for refactoring.
    """
    (
        Query(path)
        .select("decorator<'@' name='python_2_unicode_compatible' any>")
        .modify(remove_node)
        .select("import_from<'from' module_name=any 'import' 'python_2_unicode_compatible'>")
        .modify(remove_node)
        .write()
    )


@click.command()
@click.option('--path', help='use syntax: --path {path_to_input_file/directory}')
def main(path):
    """
    Function to handle input path for refactoring.
    HOW_TO_USE: when running as a repo tool, use following syntax to run the command:
        remove_python2_unicode_compatible --path {path_to_input_file/directory}
    """
    run_removal_query(path)


if __name__ == '__main__':
    main()



"""
Tests for Github Actions Modernizer Script
"""
import os
import shutil
import uuid
from unittest import TestCase

from edx_repo_tools.codemods.django3 import GithubCIModernizer
from edx_repo_tools.utils import YamlLoader


class TestGithubActionsModernizer(TestCase):

    def setUp(self):
        self.test_file1 = self._setup_local_copy("sample_files/sample_ci_file.yml")
        self.test_file2 = self._setup_local_copy("sample_files/sample_ci_file_2.yml")
        self.test_file3 = self._setup_local_copy("sample_files/sample_ci_file_3.yml")

    @staticmethod
    def _setup_local_copy(file_name):
        current_directory = os.path.dirname(__file__)
        temp_file = os.path.join(current_directory, str(uuid.uuid4()) + ".yml")
        local_file = os.path.join(current_directory, file_name)
        shutil.copy2(local_file, temp_file)
        return temp_file

    @staticmethod
    def _get_updated_yaml_elements(file_path):
        modernizer = GithubCIModernizer(file_path)
        modernizer.modernize()
        yaml_loader = YamlLoader(file_path)
        return yaml_loader.elements

    def test_python_matrix_items(self):
        ci_elements = TestGithubActionsModernizer._get_updated_yaml_elements(self.test_file1)
        python_versions = ci_elements['jobs']['run_tests']['strategy']['matrix']['python-version']

        self.assertIsInstance(python_versions, list)
        self.assertNotIn('3.5', python_versions)

    def test_python_matrix_items_build_tag(self):
        ci_elements = TestGithubActionsModernizer._get_updated_yaml_elements(self.test_file3)
        python_versions = ci_elements['jobs']['build']['strategy']['matrix']['python-version']

        self.assertIsInstance(python_versions, list)
        self.assertNotIn('3.5', python_versions)

    def test_include_exclude_list(self):
        ci_elements = TestGithubActionsModernizer._get_updated_yaml_elements(self.test_file2)
        include_list = ci_elements['jobs']['run_tests']['strategy']['matrix'].get('include', {})
        exclude_list = ci_elements['jobs']['run_tests']['strategy']['matrix'].get('exclude', {})

        for item in list(include_list) + list(exclude_list):
            self.assertNotEqual(item['python-version'], '3.5')

    def tearDown(self):
        os.remove(self.test_file1)
        os.remove(self.test_file2)
        os.remove(self.test_file3)



import click
import subprocess


@click.command()
@click.option('--path', help='use syntax: --path {path_to_input_file/directory}')
def main(path):
    """
    Function to handle input path for refactoring.
    HOW_TO_USE: when running as a repo tool, use following syntax to run the command:
        replace_staticfiles --path {path_to_input_file/directory}
    """
    subprocess.run(['./edx_repo_tools/codemods/django3/script_to_replace_static.sh', path])


if __name__ == '__main__':
    main()




import re
from os import path
import urllib.request

import click

FILES = [
    'requirements/constraint.txt',
    'requirements/constraints.txt',
    'requirements/pins.txt',
]


class CommonConstraint:
    """
        CommonConstraint class is responsible for adding common constraint pin in
        the constraints file of the repository
    """

    def __init__(self):
        self.comment = "# Common constraints for edx repos\n"
        self.constraint = "-c https://raw.githubusercontent.com/edx/edx-lint/master/edx_lint/files/common_constraints" \
                          ".txt\n "
        self.file = self._get_file_name()
        self.lines = []

    def _get_file_name(self):
        for file in FILES:
            if path.exists(file):
                return file

    def _read_lines(self):
        with open(self.file, 'r') as file:
            self.lines = file.readlines()

    def _get_constraint_index(self):
        for i in range(len(self.lines)):
            if not self.lines[i].lstrip().startswith('#'):
                if self.lines[i] == '\n':
                    return i + 1
        return 0

    def _get_constraints(self):
        target_url = "https://raw.githubusercontent.com/edx/edx-lint/master/edx_lint/files/common_constraints.txt"

        packages = []
        for raw_line in urllib.request.urlopen(target_url):
            line = raw_line.decode('utf-8')
            package = re.search('^[A-Za-z0-9-_]+(<|==|>)', line)
            if package:
                packages.append(re.sub("(<=|==|>=|>|<)([0-9]*.*)\n", "", package.string.lower()))
        return packages

    def _remove_common_constraints(self):
        constraints = self._get_constraints()
        for index, line in enumerate(self.lines):
            package = re.search('^[A-Za-z0-9-_]+(<|==|>)', line)
            if package:
                if re.sub("(<=|==|>=|>|<)([0-9]*.*)\n", "", package.string.lower()) in constraints:
                    del self.lines[index]
                    if self.lines[index - 1].lstrip().startswith('#'):
                        del self.lines[index - 1]
                    if self.lines[index - 2] == '\n':
                        del self.lines[index - 1]

    def _insert_constraint(self):
        index = self._get_constraint_index()

        self.lines.insert(index, self.comment)
        self.lines.insert(index + 1, self.constraint)
        self.lines.insert(index + 2, "\n")

        return self.lines

    def _write_file(self):
        with open(self.file, 'w') as file:
            file.writelines(self.lines)

    def update_file(self):
        if self.file is None:
            raise click.ClickException('No constraint file exists!')

        self._read_lines()
        self._insert_constraint()
        self._remove_common_constraints()
        self._write_file()

        click.echo('Added common constraint successfully!')


@click.command()
def main():
    constraint = CommonConstraint()
    constraint.update_file()


if __name__ == "__main__":
    main()



"""
Django Matrix Modernizer for Github Actions CI
"""
import re
from copy import deepcopy

import click

from edx_repo_tools.utils import YamlLoader

DJANGO_ENV_PATTERN = r"django[0-3][0-2][0-2]?"
ALLOWED_DJANGO_ENVS = ['django22', 'django30', 'django31', 'django32']


class GithubCIDjangoModernizer(YamlLoader):
    def __init__(self, file_path):
        super().__init__(file_path)
        self.yml_instance.default_flow_style = None
        self.yml_instance.indent(mapping=2, sequence=2, offset=0)

    def _update_matrix_items(self, job_name, matrix_item_name, matrix_item):
        if not isinstance(matrix_item, list):
            return

        has_django_env = any(
            re.match(DJANGO_ENV_PATTERN, item) for item in matrix_item
        )
        if not has_django_env:
            return
        non_django_matrix_items = [item for item in matrix_item if not re.match(DJANGO_ENV_PATTERN, item)]
        updated_matrix_items = non_django_matrix_items + ALLOWED_DJANGO_ENVS
        self.elements['jobs'][job_name]['strategy']['matrix'][matrix_item_name] = updated_matrix_items

    def _update_django_matrix_items(self, job_name, job):
        matrices = job.get('strategy').get('matrix').items()
        for matrix_item_key, matrix_item in matrices:
            self._update_matrix_items(job_name, matrix_item_key, matrix_item)

    def _update_job_matrices(self):
        for job_name, job in self.elements.get('jobs').items():
            self._update_django_matrix_items(job_name, job)

    def modernize(self):
        self._update_job_matrices()
        self.update_yml_file()


@click.command()
@click.option(
    '--path', default='.github/workflows/ci.yml',
    help="Path to default CI workflow file")
def main(path):
    modernizer = GithubCIDjangoModernizer(path)
    modernizer.modernize()


if __name__ == '__main__':
    main()



import sys
from lib2to3.fixer_util import Name, Comma
from typing import Optional

from bowler import Query, LN, Capture, Filename

DJANGO_SHORTCUT_FILES = []


def filter_render_function(node: LN, capture: Capture, filename: Filename) -> bool:
    import_statement = capture.get("function_import")
    function_call = capture.get("function_call")
    if import_statement:
        if "django.shortcuts" in str(import_statement.children[1]):
            DJANGO_SHORTCUT_FILES.append(filename)
            return True
    elif function_call:
        # Only rename and modify those render_to_response which are imported from django.shortcuts
        return filename in DJANGO_SHORTCUT_FILES

    return False


def add_request_param(node: LN, capture: Capture, filename: Filename) -> Optional[LN]:
    arguments = capture.get("function_arguments")
    if arguments:
        arguments = arguments[0]
        # Insert request parameter at start
        arguments.insert_child(0, Name("request"))
        arguments.insert_child(1, Comma())
        arguments.children[2].prefix = " "

    return node


def main():
    (
        Query(sys.argv[1])
            .select_function("render_to_response")
            .filter(filter_render_function)
            .rename('render')
            .modify(add_request_param)
            .write()
    )


if __name__ == '__main__':
    main()



"""
Tests for Github Actions Django Modernizer Script
"""
import os
import shutil
from os.path import dirname, basename, join

from edx_repo_tools.codemods.django3 import GithubCIDjangoModernizer
from edx_repo_tools.utils import YamlLoader


def setup_local_copy(filepath, tmpdir):
    current_directory = dirname(__file__)
    local_file = join(current_directory, filepath)
    temp_file_path = str(join(tmpdir, basename(filepath)))
    shutil.copy2(local_file, temp_file_path)
    return temp_file_path


def get_updated_yaml_elements(file_path):
    modernizer = GithubCIDjangoModernizer(file_path)
    modernizer.modernize()
    yaml_loader = YamlLoader(file_path)
    return yaml_loader.elements


def test_matrix_items(tmpdir):
    """
    Test the scenario where django env is present in the tox-envs within a single job workflow.
    """
    test_file = setup_local_copy("sample_files/sample_ci_file.yml", tmpdir)
    ci_elements = get_updated_yaml_elements(test_file)
    tox_envs = ci_elements['jobs']['run_tests']['strategy']['matrix']['toxenv']

    assert 'django30' in tox_envs
    assert 'django31' in tox_envs
    assert 'django32' in tox_envs


def test_matrix_items_multiple_jobs(tmpdir):
    """
    Test the scenarios with multiple jobs including/excluding django in test tox-envs.
    """
    test_file = setup_local_copy("sample_files/sample_ci_file_multiple_jobs.yml", tmpdir)
    ci_elements = get_updated_yaml_elements(test_file)

    # test the case with django env present in one job
    job1_tox_envs = ci_elements['jobs']['build']['strategy']['matrix']['tox-env']
    assert 'django30' in job1_tox_envs
    assert 'django31' in job1_tox_envs
    assert 'django32' in job1_tox_envs

    # test the case with django env present in second job
    job2_tox_envs = ci_elements['jobs']['django_test']['strategy']['matrix']['django-version']
    assert 'django30' in job2_tox_envs
    assert 'django31' in job2_tox_envs
    assert 'django32' in job2_tox_envs

    # test the case with no django env present in third job.
    job3_tox_envs = ci_elements['jobs']['test']['strategy']['matrix']['tox']
    assert 'django30' not in job3_tox_envs
    assert 'django31' not in job3_tox_envs
    assert 'django32' not in job3_tox_envs



"""
Tests for new django32 settings codemod
"""
import os
import shutil
from os.path import dirname, basename, join
from edx_repo_tools.codemods.django3.add_new_django32_settings import SettingsModernizer

def setup_local_copy(filepath, tmpdir):
    current_directory = dirname(__file__)
    local_file = join(current_directory, filepath)
    temp_file_path = str(join(tmpdir, basename(filepath)))
    shutil.copy2(local_file, temp_file_path)
    return temp_file_path

def test_update_existing_hashing_algorithm(tmpdir):
    """
    Test the case where an old hashing algorithm is already present in the settings
    """
    test_file = setup_local_copy("sample_files/sample_django_settings_2.py", tmpdir)
    setting_modernizer = SettingsModernizer(setting_path=test_file, is_service=True)
    setting_modernizer.update_hash_algorithm()
    with open(test_file) as test_setting_file:
        target_algorithm = f"{setting_modernizer.DEFAULT_ALGORITHM_KEY} = '{setting_modernizer.NEW_HASHING_ALGORITHM}'"
        assert target_algorithm in test_setting_file.read()

def test_add_new_hashing_algorithm(tmpdir):
    """
    Test the case where no hashing algorithm is present in the settings
    """
    test_file = setup_local_copy("sample_files/sample_django_settings.py", tmpdir)
    setting_modernizer = SettingsModernizer(setting_path=test_file, is_service=True)
    setting_modernizer.update_hash_algorithm()
    with open(test_file) as test_setting_file:
        target_algorithm = f"{setting_modernizer.DEFAULT_ALGORITHM_KEY} = '{setting_modernizer.NEW_HASHING_ALGORITHM}'"
        assert target_algorithm in test_setting_file.read()

def test_update_existing_auto_field(tmpdir):
    """
    Test the case where an old value of auto field is present in the settings
    """
    test_file = setup_local_copy("sample_files/sample_django_settings_2.py", tmpdir)
    setting_modernizer = SettingsModernizer(setting_path=test_file, is_service=True)
    setting_modernizer.update_auto_field()
    with open(test_file) as test_setting_file:
        target_field = f"{setting_modernizer.DEFAULT_FIELD_KEY} = '{setting_modernizer.NEW_AUTO_FIELD}'"
        assert target_field in test_setting_file.read()

def test_add_new_auto_field(tmpdir):
    """
    Test the case where no auto field value is present in the settings
    """
    test_file = setup_local_copy("sample_files/sample_django_settings.py", tmpdir)
    setting_modernizer = SettingsModernizer(setting_path=test_file, is_service=True)
    setting_modernizer.update_auto_field()
    with open(test_file) as test_setting_file:
        target_field = f"{setting_modernizer.DEFAULT_FIELD_KEY} = '{setting_modernizer.NEW_AUTO_FIELD}'"
        assert target_field in test_setting_file.read()

def test_add_new_context_processor(tmpdir):
    """
    Test the case where no request context_processor is present in the settings
    """
    test_file = setup_local_copy("sample_files/sample_django_settings.py", tmpdir)
    setting_modernizer = SettingsModernizer(setting_path=test_file, is_service=True)
    setting_modernizer.update_context_processors()
    with open(test_file) as test_setting_file:
        assert setting_modernizer.NEW_PROCESSOR in test_setting_file.read()



"""Tests of tag_release.py"""

from collections import OrderedDict

from github3 import GitHubError
from github3.exceptions import NotFoundError
from github3.repos.repo import Repository
from unittest.mock import Mock
import pytest

from edx_repo_tools.release.tag_release import (
    get_ref_for_repos, commit_ref_info,
    create_ref_for_repos, remove_ref_for_repos, override_repo_refs,
    TagReleaseError
)

ALREADY_EXISTS = GitHubError(Mock(status_code=422, json=Mock(return_value={"message": "Reference already exists"})))


class FakeNotFoundError(NotFoundError):
    """A NotFoundError with enough content to please the code."""
    status_code = 404
    content = "A fake response"

    def __init__(self):
        super().__init__(self)


def mock_repository(name, has_refs=False):
    """Make a mock Repository object.

    Arguments:
        name (str): will be the name and full_name of the repo.
        has_refs (bool): if False, has no refs. If True, returns mock refs.

    Returns:
        A Mock

    """
    repo = Mock(spec=Repository, name=name, full_name=name)
    if not has_refs:
        repo.ref.side_effect = FakeNotFoundError
    return repo

def find_repo_item(repos, name):
    """Find a repo in a dict mapping Repository objects to data.

    Arguments:
        repos: (dict mapping Repository objects to data)
        name: (str) the full_name of the repo to find.

    Returns:
        The data found in the dict, or None if not found.

    """
    for repo, data in repos.items():
        if repo.full_name == name:
            return repo, data
    return None, None


def find_repo(repos, name):
    """Find the key in `repos` that is Repository(`name`)"""
    return find_repo_item(repos, name)[0]


def find_repo_data(repos, name):
    """Find the value in `repos` associated with Repository(`name`)"""
    return find_repo_item(repos, name)[1]


@pytest.fixture
def expected_repos():
    """Newly created test data for each test.

    The keys are mocks, so are changed by each test, so we need new data for
    each test, not a global constant.

    """
    repos = {
        mock_repository('edx/edx-platform'): {
            'openedx-release': {
                'ref': 'release',
            }
        },
        mock_repository('edx/configuration'): {
            'openedx-release': {
                'ref': 'master',
            }
        },
        mock_repository('edx/XBlock'): {
            'openedx-release': {
                'parent-repo': 'edx/edx-platform',
            }
        },
    }
    return repos


@pytest.fixture
def expected_commits():
    """Newly created test data for each test.

    The keys are mocks, so are changed by each test, so we need new data for
    each test, not a global constant.

    """
    commits = {
        mock_repository('edx/edx-platform'): {
            'committer': {'name': 'Dev 1'},
            'author': {'name': 'Dev 1'},
            'sha': 'deadbeef12345',
            'ref_type': 'branch',
            'ref': 'release',
            'message': 'commit message for edx-platform release commit',
        },
        mock_repository('edx/configuration'): {
            'committer': {'name': 'Dev 2'},
            'author': {'name': 'Dev 2'},
            'sha': '12345deadbeef',
            'ref_type': 'branch',
            'ref': 'master',
            'message': 'commit message for configuration master commit',
        },
        mock_repository('edx/XBlock'): {
            'committer': {'name': 'Dev 1'},
            'author': {'name': 'Dev 1'},
            'sha': '1a2b3c4d5e6f',
            'ref_type': 'tag',
            'ref': '0.4.4',
            'message': 'commit message for refs/tags/0.4.4',
        }
    }
    return commits


def test_get_ref_for_repos():
    repos = [
        mock_repository('edx/edx-platform', has_refs=True),
        mock_repository('edx/configuration'),
        mock_repository('edx/XBlock'),
    ]

    result = get_ref_for_repos(repos, "tag-exists-some-repos")
    expected_result = {
        'edx/edx-platform': {
            'author': repos[0].git_commit().refresh.return_value.author,
            'committer': repos[0].git_commit().refresh.return_value.committer,
            'message': repos[0].git_commit().refresh.return_value.message,
            'ref': 'refs/tags/tag-exists-some-repos',
            'ref_type': 'tag',
            'sha': repos[0].git_commit().refresh.return_value.sha,
        }
    }
    assert result == expected_result


def test_get_ref_for_repos_not_exist():
    repos = [
        mock_repository('edx/edx-platform'),
        mock_repository('edx/configuration'),
        mock_repository('edx/XBlock'),
    ]
    for repo in repos:
        repo.ref.return_value = None
    result = get_ref_for_repos(repos, "tag-exists-no-repos")
    assert result == {}


def test_commit_ref_info(expected_repos):
    result = commit_ref_info(expected_repos)

    edx_edx_platform = find_repo(expected_repos, 'edx/edx-platform')
    edx_configuration = find_repo(expected_repos, 'edx/configuration')
    edx_edx_platform.branch.assert_called_once_with('release')
    assert not edx_edx_platform.ref.called
    edx_configuration.branch.assert_called_once_with('master')
    assert not edx_configuration.ref.called

    # The XBlock repo shouldn't have been examined at all.
    assert not find_repo(expected_repos, 'edx/XBlock').branch.called

    edx_platform_commit = edx_edx_platform.git_commit().refresh()
    configuration_commit = edx_configuration.git_commit().refresh()

    expected_commits = {
        edx_edx_platform: {
            'committer': edx_platform_commit.committer,
            'author': edx_platform_commit.author,
            'sha': edx_platform_commit.sha,
            'ref_type': 'branch',
            'ref': 'release',
            'message': edx_platform_commit.message,
        },
        edx_configuration: {
            'committer': configuration_commit.committer,
            'author': configuration_commit.author,
            'sha': configuration_commit.sha,
            'ref_type': 'branch',
            'ref': 'master',
            'message': configuration_commit.message,
        },
    }

    assert result == expected_commits


def test_overrides_none(expected_repos):
    result = override_repo_refs(expected_repos)
    assert result == expected_repos


def test_overrides_global_ref(expected_repos):
    result = override_repo_refs(expected_repos, override_ref="abcdef")
    assert find_repo_data(result, "edx/edx-platform")["openedx-release"]["ref"] == "abcdef"
    assert find_repo_data(result, "edx/configuration")["openedx-release"]["ref"] == "abcdef"


def test_overrides_dict(expected_repos):
    overrides = {
        "edx/edx-platform": "xyz",
        "edx/configuration": "refs/branch/no-way",
        "edx/does-not-exist": "does-not-matter",
    }
    result = override_repo_refs(expected_repos, overrides=overrides)
    assert find_repo_data(result, "edx/edx-platform")["openedx-release"]["ref"] == "xyz"
    assert find_repo_data(result, "edx/configuration")["openedx-release"]["ref"] == "refs/branch/no-way"


def test_overrides_global_ref_and_dict(expected_repos):
    override_ref = "fakie-mcfakerson"
    overrides = {
        "edx/edx-platform": "xyz",
        "edx/does-not-exist": "does-not-matter",
    }
    result = override_repo_refs(
        expected_repos,
        override_ref=override_ref,
        overrides=overrides,
    )
    assert find_repo_data(result, "edx/edx-platform")["openedx-release"]["ref"] == "xyz"
    assert find_repo_data(result, "edx/configuration")["openedx-release"]["ref"] == "fakie-mcfakerson"


def test_create_happy_path(expected_commits):
    # Creating a tag that does not already exist anywhere.
    result = create_ref_for_repos(
        expected_commits,
        "tag-exists-no-repos",
        dry=False
    )
    assert result is True

    find_repo(expected_commits, 'edx/edx-platform').create_ref.assert_called_once_with(
        sha="deadbeef12345",
        ref="refs/tags/tag-exists-no-repos",
    )

    find_repo(expected_commits, 'edx/configuration').create_ref.assert_called_once_with(
        sha="12345deadbeef",
        ref="refs/tags/tag-exists-no-repos",
    )

    find_repo(expected_commits, 'edx/XBlock').create_ref.assert_called_once_with(
        sha="1a2b3c4d5e6f",
        ref="refs/tags/tag-exists-no-repos",
    )


def test_create_existing_tag(expected_commits):
    # Creating a tag that already exists in edx-platform: we'll make sure
    # that edx-platform is attempted *first*, so that the function will
    # return before touching edx/configuration.
    ordered_commits = OrderedDict([
        find_repo_item(expected_commits, 'edx/edx-platform'),
        find_repo_item(expected_commits, 'edx/configuration'),
    ])

    find_repo(ordered_commits, 'edx/edx-platform').create_ref.side_effect = ALREADY_EXISTS

    # Creating a tag that does not already exist anywhere.
    result = create_ref_for_repos(
        ordered_commits,
        "tag-exists-some-repos",
        dry=False,
    )

    assert result is False
    find_repo(ordered_commits, 'edx/edx-platform').create_ref.assert_called_once_with(
        sha="deadbeef12345",
        ref="refs/tags/tag-exists-some-repos",
    )

    assert not find_repo(ordered_commits, 'edx/configuration').create_ref.called


def test_create_existing_tag_at_end(expected_commits):
    # Creating a tag that already exists in edx-platform: we'll make sure
    # that edx-platform is attempted *last* so that other repos will have
    # been touched first.
    ordered_commits = OrderedDict([
        find_repo_item(expected_commits, 'edx/configuration'),
        find_repo_item(expected_commits, 'edx/edx-platform'),
    ])

    find_repo(ordered_commits, 'edx/edx-platform').create_ref.side_effect = ALREADY_EXISTS

    result = create_ref_for_repos(
        ordered_commits,
        "tag-exists-some-repos",
        dry=False,
    )
    assert result is False

    find_repo(ordered_commits, 'edx/edx-platform').create_ref.assert_called_once_with(
        sha="deadbeef12345",
        ref="refs/tags/tag-exists-some-repos",
    )

    find_repo(ordered_commits, 'edx/configuration').create_ref.assert_called_once_with(
        sha="12345deadbeef",
        ref="refs/tags/tag-exists-some-repos",
    )

    # Second request failed, so rollback the first.
    find_repo(ordered_commits, 'edx/configuration').create_ref.return_value.delete.assert_called_once_with()


def test_create_existing_tag_at_end_no_rollback(expected_commits):
    # Creating a tag that already exists in edx-platform: we'll make sure
    # that edx-platform is attempted *last* so that other repos will have
    # been touched first.
    ordered_commits = OrderedDict([
        find_repo_item(expected_commits, 'edx/configuration'),
        find_repo_item(expected_commits, 'edx/edx-platform'),
    ])

    find_repo(ordered_commits, 'edx/edx-platform').create_ref.side_effect = ALREADY_EXISTS

    with pytest.raises(TagReleaseError) as excinfo:
        create_ref_for_repos(
            ordered_commits,
            "tag-exists-some-repos",
            rollback_on_fail=False,
            dry=False,
        )

    find_repo(ordered_commits, 'edx/edx-platform').create_ref.assert_called_once_with(
        sha="deadbeef12345",
        ref="refs/tags/tag-exists-some-repos",
    )

    find_repo(ordered_commits, 'edx/configuration').create_ref.assert_called_once_with(
        sha="12345deadbeef",
        ref="refs/tags/tag-exists-some-repos",
    )

    # Second request failed, no rollback of the first.
    assert not find_repo(ordered_commits, 'edx/configuration').create_ref.return_value.delete.called

    assert "No rollback attempted" in str(excinfo.value)
    assert "Reference already exists" in str(excinfo.value)
    assert "Refs exist on the following repos: " in str(excinfo.value)
    assert "edx/configuration" in str(excinfo.value)


def test_create_existing_tag_at_end_rollback_failure(expected_commits):
    # Creating a tag that already exists in edx-platform: we'll make sure
    # that edx-platform is attempted *last* so that other repos will have
    # been touched first.
    ordered_commits = OrderedDict([
        find_repo_item(expected_commits, 'edx/configuration'),
        find_repo_item(expected_commits, 'edx/edx-platform'),
    ])

    edx_edx_platform = find_repo(ordered_commits, 'edx/edx-platform')
    edx_edx_platform.create_ref.side_effect = ALREADY_EXISTS
    # When we try to delete the configuration tag, it will fail with a 500 error
    edx_configuration = find_repo(ordered_commits, 'edx/configuration')
    edx_configuration.create_ref.return_value.delete.side_effect = GitHubError(Mock(status_code=500))

    with pytest.raises(TagReleaseError) as excinfo:
        create_ref_for_repos(
            ordered_commits,
            "tag-exists-some-repos",
            dry=False,
        )

    edx_edx_platform.create_ref.assert_called_once_with(
        sha="deadbeef12345",
        ref="refs/tags/tag-exists-some-repos",
    )

    edx_configuration.create_ref.assert_called_once_with(
        sha="12345deadbeef",
        ref="refs/tags/tag-exists-some-repos",
    )

    # Second response failed, so try to rollback.
    # ... but configuration fails, so we get an exception
    edx_configuration.create_ref.return_value.delete.assert_called_once_with()
    assert "failed to delete ref on the following repos: edx/configuration" in str(excinfo)


def test_remove_all():
    repos = [
        mock_repository('edx/edx-platform', has_refs=True),
        mock_repository('edx/configuration', has_refs=True),
    ]
    result = remove_ref_for_repos(repos, "tag-exists-all-repos", dry=False)
    assert result is True
    for repo in repos:
        repo.ref.assert_called_once_with('tags/tag-exists-all-repos')
        ref = repo.ref.return_value
        ref.delete.assert_called_once_with()


def test_remove_some():
    repos = [
        mock_repository('edx/edx-platform', has_refs=False),
        mock_repository('edx/configuration', has_refs=True),
    ]

    result = remove_ref_for_repos(repos, "tag-exists-some-repos", dry=False)
    assert result is True
    repos[0].ref.assert_called_once_with('tags/tag-exists-some-repos')
    for repo in repos[1:]:
        repo.ref.assert_called_once_with('tags/tag-exists-some-repos')
        ref = repo.ref.return_value
        ref.delete.assert_called_once_with()


def test_remove_none():
    repos = [
        mock_repository('edx/edx-platform'),
        mock_repository('edx/configuration'),
    ]

    for repo in repos:
        repo.ref.return_value = None

    result = remove_ref_for_repos(repos, "tag-exists-no-repos", dry=False)
    assert result is False

    for repo in repos:
        repo.ref.assert_called_once_with('tags/tag-exists-no-repos')


def test_remove_with_errors():
    repos = [
        Mock(spec=Repository, full_name="edx/edx-platform"),
        Mock(spec=Repository, full_name="edx/configuration"),
    ]

    # when we try to get the edx-platform tag, it will fail with a 500 error
    repos[0].ref.side_effect = GitHubError(Mock(status_code=500))

    with pytest.raises(TagReleaseError) as excinfo:
        remove_ref_for_repos(repos, "tag-exists-all-repos", dry=False)

    for repo in repos:
        repo.ref.assert_called_once_with('tags/tag-exists-all-repos')

    for repo in repos[1:]:
        ref = repo.ref.return_value
        ref.delete.assert_called_once_with()

    assert "Failed to remove the ref from the following repos: edx/edx-platform" in str(excinfo)


@pytest.mark.parametrize("ref_prefix, use_tag, call_prefix", [
    ("", True, "tags/"),
    ("", False, "heads/"),
    ("tags/", True, "tags/"),
    ("tags/", False, "tags/"),
    ("refs/tags/", True, "tags/"),
    ("refs/tags/", False, "tags/"),
    ("heads/", True, "heads/"),
    ("heads/", False, "heads/"),
    ("refs/heads/", True, "heads/"),
    ("refs/heads/", False, "heads/"),
])
def test_remove_ref_formatting(ref_prefix, use_tag, call_prefix):
    repos = [
        Mock(spec=Repository, full_name="edx/edx-platform"),
        Mock(spec=Repository, full_name="edx/configuration"),
    ]
    result = remove_ref_for_repos(repos, f"{ref_prefix}tag-exists-all-repos", use_tag=use_tag, dry=False)
    assert result is True
    for repo in repos:
        repo.ref.assert_called_once_with(f'{call_prefix}tag-exists-all-repos')
        ref = repo.ref.return_value
        ref.delete.assert_called_once_with()



from .tox_modernizer import ConfigReader, ToxModernizer
from .travis_modernizer import TravisModernizer
from .travis_modernizer import DJANGO_PATTERN
from .github_actions_modernizer import GithubCIModernizer
from .github_actions_modernizer_django import GithubCIDjangoModernizer



"""
Codemod to add new settings in the repo settings file.
"""
import json
import re
import os
import click
from copy import deepcopy


class SettingsModernizer:
    """
    Django32 modernizer for updating settings files.
    """
    DEFAULT_ALGORITHM_KEY = "DEFAULT_HASHING_ALGORITHM"
    NEW_HASHING_ALGORITHM = "sha1"
    DEFAULT_FIELD_KEY = "DEFAULT_AUTO_FIELD"
    NEW_AUTO_FIELD = "django.db.models.AutoField"
    NEW_PROCESSOR = "django.template.context_processors.request"

    def __init__(self, setting_path, is_service):
        self.settings_path = setting_path
        self.is_service = is_service

    def _apply_regex_operations(self, matching_pattern, new_pattern, context_processors=False):
        file_data = open(self.settings_path).read()
        if context_processors:
            if self.NEW_PROCESSOR not in file_data and re.search("context_processors", file_data):
                file_data = re.sub(pattern=matching_pattern, repl=new_pattern, string=file_data)
        else:
            file_data = re.sub(pattern=matching_pattern, repl="", string=file_data)
            file_data = file_data+new_pattern
        return deepcopy(file_data)

    def _update_settings_file(self, matching_pattern, new_pattern, context_processors=False):
        file_data = self._apply_regex_operations(matching_pattern, new_pattern, context_processors)
        with open(self.settings_path, 'w') as setting_file:
            setting_file.write(file_data)

    def update_settings(self):
        if self.is_service:
            self.update_hash_algorithm()
        self.update_auto_field()
        self.update_context_processors()

    def update_hash_algorithm(self):
        """
        Update the HASHING_ALGORITHM in the settings file.
        """
        matching_algorithm = f"{self.DEFAULT_ALGORITHM_KEY}\s=\s'[a-zA-Z0-9]*'\\n"
        new_algorithm = f"{self.DEFAULT_ALGORITHM_KEY} = '{self.NEW_HASHING_ALGORITHM}'\n"
        self._update_settings_file(matching_algorithm, new_algorithm)

    def update_auto_field(self):
        """
        Update the AUTO_FIELD in the settings file.
        """
        matching_field = f"{self.DEFAULT_FIELD_KEY}\s=\s'([a-zA-Z](.[a-zA-Z])?)*'\\n"
        new_field = f"{self.DEFAULT_FIELD_KEY} = '{self.NEW_AUTO_FIELD}'\n"
        self._update_settings_file(matching_field, new_field)

    def update_context_processors(self):
        """
        Update the CONTEXT_PROCESSORS in the settings file.
        """
        matching_pattern = fr"'context_processors': \(([^)]*)\)"
        new_pattern = fr"'context_processors': (\1" + f"\t'{self.NEW_PROCESSOR}',\n\t\t\t\t)"
        self._update_settings_file(matching_pattern, new_pattern, context_processors=True)


@click.command()
@click.option('--setting_path', default='settings/base.py', help="Path to the settings file to update")
@click.option('--is_service', default=False, help="Flag for service/library, pass True for service")
def main(setting_path, is_service):
    setting_modernizer = SettingsModernizer(setting_path, is_service)
    setting_modernizer.update_settings()


if __name__ == '__main__':
    main()
